@@ -5,10 +5,10 @@ from arangodb.index.api import Index
 from arangodb.index.general import BaseIndex
 from arangodb.orm.fields import ModelField
 from arangodb.query.advanced import Query, Traveser
-from arangodb.query.simple import SimpleQuery
+from arangodb.query.simple import SimpleQuery, SimpleIndexQuery
 
 
-class CollectionQueryset(object):
+class LazyQueryset(object):
     """
     """
 
@@ -18,6 +18,171 @@ class CollectionQueryset(object):
 
         self._manager = manager
         self._collection = manager._model_class.collection_instance
+
+        # Cache
+        self._has_cache = False
+        self._cache = []
+
+    def _generate_cache(self):
+        """
+        """
+
+        self._cache = [] # TODO: Check how to best clear this list
+        self._has_cache = True
+
+    def __getitem__(self, item):
+        """
+            Is used for the index access
+        """
+
+        if not self._has_cache:
+            self._generate_cache()
+
+        return self._cache[item]
+
+    def __len__(self):
+        """
+        """
+
+        if not self._has_cache:
+            self._generate_cache()
+
+        return len(self._cache)
+
+
+class IndexQueryset(LazyQueryset):
+    """
+    """
+
+    def __init__(self, manager):
+        """
+        """
+
+        super(IndexQueryset, self).__init__(manager=manager)
+
+        self._index = None
+        self._filters = {}
+
+    def set_index(self, index):
+        """
+        """
+
+        self._has_cache = False
+
+        self._index = index
+
+    def filter(self, **kwargs):
+        """
+        """
+
+        self._has_cache = False
+
+        for arg_name, arg_value in kwargs.items():
+            self._filters[arg_name] = arg_value
+
+        return self
+
+    def _generate_cache(self):
+        """
+        """
+
+        super(IndexQueryset, self)._generate_cache()
+
+        index_field = getattr(self._manager._model_class, self._index)
+        result = None
+
+        # All have these attributes
+        if 'skip' in self._filters:
+            skip = self._filters['skip']
+        else:
+            skip = None
+
+        if 'limit' in self._filters:
+            limit = self._filters['limit']
+        else:
+            limit = None
+
+        # Hash index
+        if index_field.index_type_obj.type_name == 'hash':
+
+            result = SimpleIndexQuery.get_by_example_hash(
+                collection=index_field.collection,
+                index_id=index_field.index_type_obj.id,
+                example_data=self._filters,
+                allow_multiple=True,
+                skip=skip,
+                limit=limit,
+            )
+
+        # Skiplist index
+        if index_field.index_type_obj.type_name == 'skiplist':
+
+            range_query = 'left' in self._filters
+            range_query = range_query and 'right' in self._filters
+            range_query = range_query and 'closed' in self._filters
+            range_query = range_query and 'attribute' in self._filters
+
+            # Range query
+            if range_query:
+                result = SimpleIndexQuery.range(
+                    collection=index_field.collection,
+                    index_id=index_field.index_type_obj.id,
+                    attribute=self._filters['attribute'],
+                    left=self._filters['left'],
+                    right=self._filters['right'],
+                    closed=self._filters['closed'],
+                    skip=skip,
+                    limit=limit,
+                )
+
+            # Normal search query
+            else:
+                result = SimpleIndexQuery.get_by_example_skiplist(
+                    collection=index_field.collection,
+                    index_id=index_field.index_type_obj.id,
+                    example_data=self._filters,
+                    allow_multiple=True,
+                    skip=skip,
+                    limit=limit,
+                )
+
+        # Fulltext index
+        if index_field.index_type_obj.type_name == 'fulltext':
+
+            result = SimpleIndexQuery.fulltext(
+                collection=index_field.collection,
+                index_id=index_field.index_type_obj.id,
+                attribute=self._filters['attribute'],
+                example_text=self._filters['example_text'],
+                skip=skip,
+                limit=limit,
+            )
+
+        # Cap constraint
+        if index_field.index_type_obj.type_name == 'cap':
+            pass
+
+        # Geo index
+        if index_field.index_type_obj.type_name == 'geo':
+            pass
+
+        # Save cache
+        if isinstance(result, list):
+            self._cache = result
+        else:
+            self._cache.append(result)
+
+
+class CollectionQueryset(LazyQueryset):
+    """
+    """
+
+    def __init__(self, manager):
+        """
+        """
+
+        super(CollectionQueryset, self).__init__(manager=manager)
+
         self._query = Query()
         # Relations
         self._is_working_on_relations = False
@@ -25,9 +190,6 @@ class CollectionQueryset(object):
         self._relations_end_model = None
         self._relations_relation_collection = None
         self._related_model_class = None
-        # Cache
-        self._has_cache = False
-        self._cache = []
 
     def get_field_relations(self, relation_collection, related_model_class, start_model=None, end_model=None):
         """
@@ -114,8 +276,7 @@ class CollectionQueryset(object):
         """
         """
 
-        self._cache = [] # TODO: Check how to best clear this list
-        self._has_cache = True
+        super(CollectionQueryset, self)._generate_cache()
 
         if self._is_working_on_relations:
 
@@ -147,25 +308,6 @@ class CollectionQueryset(object):
             model = self._manager._create_model_from_doc(doc=doc, model_class=result_class)
             self._cache.append(model)
 
-    def __getitem__(self, item):
-        """
-            Is used for the index access
-        """
-
-        if not self._has_cache:
-            self._generate_cache()
-
-        return self._cache[item]
-
-    def __len__(self):
-        """
-        """
-
-        if not self._has_cache:
-            self._generate_cache()
-
-        return len(self._cache)
-
 class CollectionModelManager(object):
 
     def __init__(self, cls):
@@ -227,6 +369,41 @@ class CollectionModelManager(object):
         queryset = CollectionQueryset(manager=self)
         return queryset.all().exclude(**kwargs)
 
+    def search_by_index(self, index, **kwargs):
+        """
+        """
+
+        queryset = IndexQueryset(manager=self)
+        queryset.set_index(index=index)
+
+        return queryset.filter(**kwargs)
+
+    def search_in_range(self, index, attribute, left, right, closed, **kwargs):
+        """
+        """
+
+        queryset = IndexQueryset(manager=self)
+        queryset.set_index(index=index)
+
+        kwargs['attribute'] = attribute
+        kwargs['left'] = left
+        kwargs['right'] = right
+        kwargs['closed'] = closed
+
+        return queryset.filter(**kwargs)
+
+    def search_fulltext(self, index, attribute, example_text, **kwargs):
+        """
+        """
+
+        queryset = IndexQueryset(manager=self)
+        queryset.set_index(index=index)
+
+        kwargs['attribute'] = attribute
+        kwargs['example_text'] = example_text
+
+        return queryset.filter(**kwargs)
+
     def _create_model_from_doc(self, doc, model_class=None):
         """
         """
